@@ -60,7 +60,7 @@ struct _CcWifiPanel
   GtkStack           *main_stack;
   GtkWidget          *spinner;
   GtkStack           *stack;
-  GtkImage           *wifi_qr_image;
+  GtkPicture         *wifi_qr_image;
   CcQrCode           *qr_code;
 
   NMClient           *client;
@@ -355,12 +355,12 @@ wifi_panel_update_qr_image_cb (CcWifiPanel *self)
       str = get_qr_string_for_hotspot (self->client, hotspot);
       if (cc_qr_code_set_text (self->qr_code, str))
         {
-          cairo_surface_t *surface;
+          GdkPaintable *paintable;
           gint scale;
 
           scale = gtk_widget_get_scale_factor (GTK_WIDGET (self->wifi_qr_image));
-          surface = cc_qr_code_get_surface (self->qr_code, QR_IMAGE_SIZE, scale);
-          gtk_image_set_from_surface (self->wifi_qr_image, surface);
+          paintable = cc_qr_code_get_paintable (self->qr_code, QR_IMAGE_SIZE * scale);
+          gtk_picture_set_paintable (self->wifi_qr_image, paintable);
         }
     }
 
@@ -426,10 +426,10 @@ remove_wifi_device (CcWifiPanel *self,
 
   /* Destroy all stack pages related to this device */
   child = gtk_stack_get_child_by_name (self->stack, id);
-  gtk_widget_destroy (child);
+  gtk_stack_remove (self->stack, child);
 
   child = gtk_stack_get_child_by_name (self->header_stack, id);
-  gtk_widget_destroy (child);
+  gtk_stack_remove (self->header_stack, child);
 
   /* Update the title widget */
   update_devices_names (self);
@@ -565,7 +565,7 @@ update_devices_names (CcWifiPanel *self)
       if (single_page_widget)
         {
           g_object_ref (single_page_widget);
-          gtk_container_remove (GTK_CONTAINER (self->center_stack), single_page_widget);
+          gtk_stack_remove (self->center_stack, single_page_widget);
           g_object_unref (single_page_widget);
         }
 
@@ -840,24 +840,39 @@ on_stack_visible_child_changed_cb (GtkStack    *stack,
           self->spinner_binding = g_object_bind_property (net_device,
                                                           "scanning",
                                                           self->spinner,
-                                                          "active",
+                                                          "spinning",
                                                           G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
           break;
         }
     }
 }
 
+static void
+on_stop_hotspot_dialog_response_cb (GtkDialog   *dialog,
+                                    gint         response,
+                                    CcWifiPanel *self)
+{
+  if (response == GTK_RESPONSE_OK)
+    {
+      NetDeviceWifi *child;
+
+      child = NET_DEVICE_WIFI (gtk_stack_get_visible_child (self->stack));
+      net_device_wifi_turn_off_hotspot (child);
+    }
+
+  gtk_window_destroy (GTK_WINDOW (dialog));
+}
+
 static void
 hotspot_stop_clicked_cb (CcWifiPanel *self)
 {
   GtkWidget *dialog;
-  GtkWidget *window;
-  int response;
+  GtkNative *native;
 
   g_assert (CC_IS_WIFI_PANEL (self));
 
-  window = gtk_widget_get_toplevel (GTK_WIDGET (self));
-  dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+  native = gtk_widget_get_native (GTK_WIDGET (self));
+  dialog = gtk_message_dialog_new (GTK_WINDOW (native),
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_OTHER,
                                    GTK_BUTTONS_NONE,
@@ -867,17 +882,9 @@ hotspot_stop_clicked_cb (CcWifiPanel *self)
                           _("_Stop Hotspot"), GTK_RESPONSE_OK,
                           NULL);
 
-  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  g_signal_connect (dialog, "response", G_CALLBACK (on_stop_hotspot_dialog_response_cb), self);
+  gtk_window_present (GTK_WINDOW (dialog));
 
-  if (response == GTK_RESPONSE_OK)
-    {
-      NetDeviceWifi *child;
-
-      child = NET_DEVICE_WIFI (gtk_stack_get_visible_child (self->stack));
-      net_device_wifi_turn_off_hotspot (child);
-    }
-
-  gtk_widget_destroy (dialog);
 }
 
 /* Overrides */
@@ -888,28 +895,6 @@ cc_wifi_panel_get_help_uri (CcPanel *panel)
   return "help:gnome-help/net-wireless";
 }
 
-static GtkWidget *
-cc_wifi_panel_get_title_widget (CcPanel *panel)
-{
-  CcWifiPanel *self = CC_WIFI_PANEL (panel);
-
-  return GTK_WIDGET (self->center_stack);
-}
-
-static void
-cc_wifi_panel_constructed (GObject *object)
-{
-  CcWifiPanel *self;
-  CcShell *shell;
-
-  self = CC_WIFI_PANEL (object);
-  shell = cc_panel_get_shell (CC_PANEL (object));
-
-  G_OBJECT_CLASS (cc_wifi_panel_parent_class)->constructed (object);
-
-  cc_shell_embed_widget_in_header (shell, GTK_WIDGET (self->header_stack), GTK_POS_RIGHT);
-}
-
 static void
 cc_wifi_panel_finalize (GObject *object)
 {
@@ -1000,9 +985,7 @@ cc_wifi_panel_class_init (CcWifiPanelClass *klass)
   CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
   panel_class->get_help_uri = cc_wifi_panel_get_help_uri;
-  panel_class->get_title_widget = cc_wifi_panel_get_title_widget;
 
-  object_class->constructed = cc_wifi_panel_constructed;
   object_class->finalize = cc_wifi_panel_finalize;
   object_class->get_property = cc_wifi_panel_get_property;
   object_class->set_property = cc_wifi_panel_set_property;
@@ -1030,6 +1013,8 @@ cc_wifi_panel_class_init (CcWifiPanelClass *klass)
 static void
 cc_wifi_panel_init (CcWifiPanel *self)
 {
+  g_autoptr(GtkCssProvider) provider = NULL;
+
   g_resources_register (cc_network_get_resource ());
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -1079,4 +1064,11 @@ cc_wifi_panel_init (CcWifiPanel *self)
 
   /* Handle comment-line arguments after loading devices */
   handle_argv (self);
+
+  /* use custom CSS */
+  provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_resource (provider, "/org/gnome/control-center/network/wifi-panel.css");
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 }
